/*	$OpenBSD: setjmp.h,v 1.1 1998/08/28 19:56:05 mickey Exp $	*/

/*
 * machine/setjmp.h: machine dependent setjmp-related information.
 */

#define _JBLEN  48              /* size, in longs, of a jmp_buf */
